@@ -2,6 +2,7 @@ from python_helper import Constant as c
 from python_helper import log, ReflectionHelper, ObjectHelper, StringHelper
 
 ENUM_VALUE_AS_STRING_KEY = 'enumValue'
+ENUM_NAME_AS_STRING_KEY = 'enumName'
 ENUM_AS_STRING_KEY = 'enum'
 
 MAP_METHOD_NAME = 'map'
@@ -25,7 +26,10 @@ class EnumItem :
         return self if not ReflectionHelper.hasAttributeOrMethod(self, ENUM_VALUE_AS_STRING_KEY) else self.enumValue
 
     def __str__(self) :
-        return c.NOTHING.join(StringHelper.prettyPython(self.__enumItemEqMap__, tabCount=2))
+        return c.NOTHING.join(StringHelper.prettyPython(self.__enumItemEqMap__, tabCount=2, quote=c.NOTHING))
+
+    def __repr__(self) :
+        return __str__()
 
     def __eq__(self, other) :
         if isinstance(other, EnumItem) :
@@ -39,15 +43,33 @@ class EnumItem :
 
 class EnumItemStr(str) :
     ...
-
 class EnumItemInt(int) :
     ...
-
+class EnumItemFloat(float) :
+    ...
+class EnumItemDict(dict) :
+    ...
+class EnumItemSet(set) :
+    ...
+class EnumItemTuple(tuple) :
+    ...
+class EnumItemList(list) :
+    ...
 ENUM_ITEM_CLASS_DICTIONARY = {
     str : EnumItemStr,
     EnumItemStr : EnumItemStr,
     int : EnumItemInt,
-    EnumItemInt : EnumItemInt
+    EnumItemInt : EnumItemInt,
+    float : EnumItemFloat,
+    EnumItemFloat : EnumItemFloat,
+    dict : EnumItemDict,
+    EnumItemDict : EnumItemDict,
+    set : EnumItemSet,
+    EnumItemSet : EnumItemSet,
+    tuple : EnumItemTuple,
+    EnumItemTuple : EnumItemTuple,
+    list : EnumItemList,
+    EnumItemList : EnumItemList
 }
 
 def isEnumItem(possibleEnumItem) :
@@ -60,10 +82,10 @@ def isEnumItem(possibleEnumItem) :
 
 def Enum(instanceLog=False, associateReturnsTo = ENUM_VALUE_AS_STRING_KEY) :
     def Wrapper(OuterEnum, *args, **kwargs):
-        def __raiseBadImplementation__(enumValue):
-            exception = Exception(f'"{str(enumValue)}" enum value not implemented in {OuterEnum.__name__} Enum')
-            log.error(Enum, 'Bad implementation', exception)
-            raise exception
+        def __raiseBadImplementation__(enumValue, enum=None) :
+            raise Exception(f'Not possible to implement "{enumValue}" enum value in {StringHelper.prettyPython(enum, quote=c.NOTHING)} enum')
+        def __raiseEnumValueNotImplemented__(enumValue, enumClass=None, enumEqList=None) :
+            raise Exception(f'Not possible to retrieve "{enumValue}" enum value from {ReflectionHelper.getName(enumClass)}: {StringHelper.prettyPython(enumEqList, quote=c.NOTHING)} enum')
         if instanceLog :
             log.log(Enum,f'''wrapping {OuterEnum.__name__}''')
         class InnerEnum(OuterEnum, EnumClass):
@@ -80,7 +102,7 @@ def Enum(instanceLog=False, associateReturnsTo = ENUM_VALUE_AS_STRING_KEY) :
                     log.prettyPython(OuterEnum, 'attributeDataList', attributeDataList)
                 for attribute, enumValue in attributeDataList :
                     if enumValue not in ENUM_STATIC_METHOD_NAME_LIST and enumValue not in originalClassAttributeValueList :
-                        __raiseBadImplementation__(enumValue)
+                        __raiseBadImplementation__(enumValue, enum=self)
                     nativeClassDataList = []
                     attributeAttributeDataList = ReflectionHelper.getAttributeDataList(attribute)
                     if instanceLog :
@@ -92,6 +114,7 @@ def Enum(instanceLog=False, associateReturnsTo = ENUM_VALUE_AS_STRING_KEY) :
                     else :
                         ReflectionHelper.setAttributeOrMethod(attribute, ENUM_VALUE_AS_STRING_KEY, ENUM_ITEM_CLASS_DICTIONARY.get(type(association))(association))
                         nativeClassDataList = ReflectionHelper.getAttributeDataList(type(association)())
+                    ReflectionHelper.setAttributeOrMethod(attribute, ENUM_NAME_AS_STRING_KEY, enumValue)
                     if not c.TYPE_FUNCTION == ReflectionHelper.getClassName(attribute) :
                         attribute.enumValue.__enumItemMap__ = attribute.__enumItemMap__
                         attribute.enumValue.__enumItemEqMap__ = attribute.__enumItemEqMap__
@@ -104,13 +127,21 @@ def Enum(instanceLog=False, associateReturnsTo = ENUM_VALUE_AS_STRING_KEY) :
                         if not nativeData :
                             ReflectionHelper.setAttributeOrMethod(attribute.enumValue, dataName, dataInstance)
                     ReflectionHelper.setAttributeOrMethod(attribute.enumValue, ENUM_VALUE_AS_STRING_KEY, ReflectionHelper.getAttributeOrMethod(attribute, self.__associateReturnsTo__, muteLogs=True))
+                    ReflectionHelper.setAttributeOrMethod(attribute.enumValue, ENUM_NAME_AS_STRING_KEY, enumValue)
                     ReflectionHelper.setAttributeOrMethod(attribute.enumValue, ENUM_AS_STRING_KEY, self)
                 if ObjectHelper.isEmpty(OuterEnum.__enumEqList__) :
                     for attribute, value in attributeDataList :
                         if str(attribute.enumValue) not in ENUM_STATIC_METHOD_NAME_LIST :
                             valueAsString = str(attribute.enumValue)
-                            self.__enumMap__[valueAsString] = attribute
-                            self.__enumEqList__.append(valueAsString)
+                            if valueAsString not in self.__enumMap__ :
+                                self.__enumMap__[valueAsString] = attribute
+                            else :
+                                __raiseBadImplementation__(valueAsString, enum=self)
+                            if value not in self.__enumMap__ :
+                                self.__enumMap__[value] = attribute
+                            elif not self.__enumMap__[value] == attribute :
+                                __raiseBadImplementation__(valueAsString, enum=self)
+                            self.__enumEqList__.append(f'{value}({self.__associateReturnsTo__}:{valueAsString})')
 
             def __eq__(self, other) :
                 return str(self) == str(other)
@@ -119,17 +150,17 @@ def Enum(instanceLog=False, associateReturnsTo = ENUM_VALUE_AS_STRING_KEY) :
                 return str(self) != str(other)
 
             def __str__(self) :
-                return c.NOTHING.join([ReflectionHelper.getName(OuterEnum), StringHelper.prettyPython(self.__enumEqList__)])
+                return c.NOTHING.join([ReflectionHelper.getName(OuterEnum), c.COLON_SPACE, StringHelper.prettyPython(self.__enumEqList__, quote=c.NOTHING)])
+
+            def __repr__(self) :
+                return __str__()
 
             @staticmethod
             def map(enumItemOrEnumItemValue) :
                 if instanceLog :
                     log.log(OuterEnum,f'''enumItemOrEnumItemValue: {enumItemOrEnumItemValue}''')
-                if enumItemOrEnumItemValue in OuterEnum.__enumMap__ :
-                    return enumItemOrEnumItemValue
-                else :
-                    log.prettyPython(OuterEnum, 'OuterEnum.__enumMap__', OuterEnum.__enumMap__)
-                    return OuterEnum.__enumMap__.get(enumItemOrEnumItemValue, __raiseBadImplementation__(enumItemOrEnumItemValue))
+                mappedEnum = OuterEnum.__enumMap__.get(str(enumItemOrEnumItemValue))
+                return mappedEnum if ObjectHelper.isNotNone(mappedEnum) else __raiseEnumValueNotImplemented__(enumItemOrEnumItemValue, enumClass=OuterEnum, enumEqList=OuterEnum().__enumEqList__)
         ReflectionHelper.overrideSignatures(InnerEnum, OuterEnum)
         return InnerEnum
     return Wrapper
